/**
 * Simple class to hold a piece of data and its count. The class has package
 * access so that the various implementations of DataCounter can access its
 * contents.  See also the return type of the getIterator method of DataCounter.
 * 
 * @param <E> type of data whose count we are recording.
 */
class DataCount<E> {
    /**
     * The data element whose count we are recording.
     */
    E data;

    /**
     * The count for the data element.
     */
    int count;

    /**
     * Create a new data count.
     * 
     * @param data the data element whose count we are recording.
     * @param count the count for the data element.
     */
    DataCount(E data, int count) {
        this.data = data;
        this.count = count;
    }
}

/**
 * Interface for a data structure that allows you to count the number of times
 * you see each piece of data.
 * 
 * Although you will be using this interface only with Strings, we have 
 * made the code generic and your data structures will follow the "function object"
 * approach.  This approach is important when you do HashTable because you need
 * to implement your own hashcode but the String class is final.
 *
 * @param <E> The type of data to be counted.
 */
public interface DataCounter<E> {

    /**
     * Increment the count for a particular data element.
     * 
     * @param data data element whose count to increment.
     */
    public void incCount(E data);

    /**
     * The number of unique data elements in the structure.
     * 
     * @return the number of unique data elements in the structure.
     */
    public int getSize();
   
    /**
     * The current count for the data, 0 if it is not in the counter.
     */
    public int getCount(E data);
    
    /**
     * Clients must not increment counts between an iterator's creation and its
     * final use.  Data structures need not check this.
     * 
     * @return an iterator for the elements.
     */
    public SimpleIterator<DataCount<E>> getIterator();
}
